package net.sf.varscan;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.RandomAccessFile;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.HashMap;
import net.sf.varscan.CallMpileup;
import net.sf.varscan.CallPileup;
import net.sf.varscan.Comparison;
import net.sf.varscan.CopyCaller;
import net.sf.varscan.Copynumber;
import net.sf.varscan.Coverage;
import net.sf.varscan.FilterSomatic;
import net.sf.varscan.FilterVariants;
import net.sf.varscan.FishersExact;
import net.sf.varscan.FpFilter;
import net.sf.varscan.LimitVariants;
import net.sf.varscan.ProcessSomatic;
import net.sf.varscan.ReadCounts;
import net.sf.varscan.Somatic;
import net.sf.varscan.Trio;

public class VarScan {

   static final double MIN_FREQ_FOR_HOM = 0.7D;


   public static void main(String[] args) {
      String usage = "VarScan v2.3\n\nUSAGE: java -jar VarScan.jar [COMMAND] [OPTIONS] \n\n";
      usage = usage + "COMMANDS:\n" + "\tpileup2snp\t\tIdentify SNPs from a pileup file\n" + "\tpileup2indel\t\tIdentify indels a pileup file\n" + "\tpileup2cns\t\tCall consensus and variants from a pileup file\n" + "\tmpileup2snp\t\tIdentify SNPs from an mpileup file\n" + "\tmpileup2indel\t\tIdentify indels an mpileup file\n" + "\tmpileup2cns\t\tCall consensus and variants from an mpileup file\n\n" + "\tsomatic\t\t\tCall germline/somatic variants from tumor-normal pileups\n" + "\tcopynumber\t\t\tDetermine relative tumor copy number from tumor-normal pileups\n" + "\treadcounts\t\tObtain read counts for a list of variants from a pileup file\n\n" + "\tfilter\t\t\tFilter SNPs by coverage, frequency, p-value, etc.\n" + "\tsomaticFilter\t\tFilter somatic variants for clusters/indels\n" + "\tfpfilter\t\tApply the false-positive filter\n\n" + "\tprocessSomatic\t\tIsolate Germline/LOH/Somatic calls from output\n" + "\tcopyCaller\t\tGC-adjust and process copy number changes from VarScan copynumber output\n" + "\tcompare\t\t\tCompare two lists of positions/variants\n" + "\tlimit\t\t\tRestrict pileup/snps/indels to ROI positions\n" + "\n";
      if(args.length > 0) {
         HashMap params = getParams(args);
         if(args[0].equals("pileup2snp")) {
            pileup2call(args, params, "SNP");
         } else if(args[0].equals("pileup2indel")) {
            pileup2call(args, params, "INDEL");
         } else if(args[0].equals("pileup2cns")) {
            pileup2call(args, params, "CNS");
         } else if(!args[0].equals("mpileup2snp") && !args[0].equals("mpileup2indel") && !args[0].equals("mpileup2cns") && !args[0].equals("mpileup2vcf")) {
            if(args[0].equals("filter")) {
               filter(args, params);
            } else if(args[0].equals("somaticFilter")) {
               somaticFilter(args, params);
            } else if(args[0].equals("fpfilter")) {
               fpfilter(args, params);
            } else if(args[0].equals("processSomatic")) {
               processSomatic(args, params);
            } else if(args[0].equals("copyCaller")) {
               copyCaller(args, params);
            } else if(args[0].equals("compare")) {
               compare(args, params);
            } else if(args[0].equals("readcounts")) {
               readcounts(args, params);
            } else if(args[0].equals("somatic")) {
               somatic(args, params);
            } else if(args[0].equals("trio")) {
               trio(args, params, "CNS");
            } else if(args[0].equals("copynumber")) {
               copynumber(args, params);
            } else if(args[0].equals("limit")) {
               limit(args, params);
            } else if(args[0].equals("coverage")) {
               coverage(args, params);
            } else if(args[0].equals("test")) {
               System.err.println("Testing...");

               try {
                  RandomAccessFile e = new RandomAccessFile("test.fasta", "r");
                  e.seek(52L);
                  byte[] buffer = new byte[5];
                  e.read(buffer);
                  String thisBase = new String(buffer);
                  System.err.println("Got " + thisBase);
               } catch (Exception var6) {
                  System.err.println("Error: Reference file: " + var6.getLocalizedMessage());
               }
            } else {
               System.err.println("Command not recognized\n" + usage);
            }
         } else {
            mpileup2call(args, params, "CNS");
         }
      } else {
         System.err.println(usage);
      }

   }

   public static void pileup2call(String[] args, HashMap params, String callType) {
      new CallPileup(args, callType);
   }

   public static void mpileup2call(String[] args, HashMap params, String callType) {
      new CallMpileup(args, callType);
   }

   public static void readcounts(String[] args, HashMap params) {
      new ReadCounts(args, params);
   }

   public static void somatic(String[] args, HashMap params) {
      if(params.containsKey("mpileup")) {
         new Somatic(args, true);
      } else {
         new Somatic(args);
      }

   }

   public static void trio(String[] args, HashMap params, String callType) {
      new Trio(args, callType);
   }

   public static void copynumber(String[] args, HashMap params) {
      if(params.containsKey("mpileup")) {
         new Copynumber(args, true);
      } else {
         new Copynumber(args);
      }

   }

   public static void filter(String[] args, HashMap params) {
      new FilterVariants(args);
   }

   public static void fpfilter(String[] args, HashMap params) {
      new FpFilter(args);
   }

   public static void somaticFilter(String[] args, HashMap params) {
      new FilterSomatic(args);
   }

   public static void processSomatic(String[] args, HashMap params) {
      new ProcessSomatic(args);
   }

   public static void copyCaller(String[] args, HashMap params) {
      new CopyCaller(args, params);
   }

   public static void compare(String[] args, HashMap params) {
      new Comparison(args);
   }

   public static void limit(String[] args, HashMap params) {
      new LimitVariants(args);
   }

   public static void coverage(String[] args, HashMap params) {
      new Coverage(args);
   }

   static HashMap getParams(String[] args) {
      HashMap params = new HashMap();
      String arg = "";
      String value = "";
      int i = 0;

      for(boolean j = false; i < args.length; ++i) {
         int var6 = i + 1;
         arg = args[i];
         if(arg.startsWith("-")) {
            while(arg.startsWith("-")) {
               arg = arg.replaceFirst("-", "");
            }

            if(i < args.length && var6 < args.length && !args[var6].startsWith("-")) {
               value = args[var6];
               params.put(arg, value);
            } else {
               params.put(arg, "true");
            }
         }
      }

      return params;
   }

   static BufferedReader getInfile(String[] args) {
      BufferedReader in = null;

      try {
         if(args.length > 1 && !args[1].startsWith("-")) {
            File instream = new File(args[1]);
            if(instream.exists()) {
               System.err.println("Reading input from " + args[1]);
               in = new BufferedReader(new FileReader(args[1]));
            }
         }

         if(in == null) {
            InputStreamReader var6 = new InputStreamReader(System.in);
            Thread.sleep(1000L);
            int num_naps = 0;

            while(!var6.ready()) {
               System.err.println("Input stream not ready, waiting for 5 seconds...");
               Thread.sleep(5000L);
               ++num_naps;
               if(num_naps >= 100) {
                  System.err.println("ERROR: Gave up waiting after 500 seconds...\n");
                  System.exit(10);
               }
            }

            if(var6.ready()) {
               System.err.println("Reading input from STDIN");
               in = new BufferedReader(var6);
            }
         }
      } catch (Exception var5) {
         System.err.println("ERROR: Unable to open input stream\n");
         System.exit(10);
      }

      return in;
   }

   static HashMap getReadCounts(String refBase, String readBases, String readQuals, int minAvgQual, String mapQuals) {
      HashMap readCounts = new HashMap();
      HashMap readCountsPlus = new HashMap();
      HashMap readCountsMinus = new HashMap();
      HashMap qualitySum = new HashMap();
      HashMap mapQualitySum = new HashMap();
      HashMap strandsSeen = new HashMap();
      int reads1 = 0;
      int reads1indel = 0;
      String readBase = "";
      String prevBase = "";
      String nextBase = "";
      int baseQuality = 0;
      int prevBaseQuality = 0;
      int mapQuality = 1;
      String strand = "";
      String[] arrBases = readBases.split("");
      char[] arrQualities = readQuals.toCharArray();
      char[] mapQualities = mapQuals.toCharArray();
      boolean readStart = false;
      int j = 0;

      String key;
      int var44;
      int var47;
      for(int results = 0; results < arrBases.length; ++results) {
         readBase = arrBases[results];
         if(results == 0 && readBase.length() == 0) {
            ++results;
            readBase = arrBases[results];
         }

         prevBase = "";
         if(results > 1 && results < arrBases.length - 1) {
            prevBase = arrBases[results - 1];
         }

         if(j > 1 && j < arrQualities.length - 1) {
            prevBaseQuality = arrQualities[j - 1] - 33;
         }

         nextBase = "";
         if(results < arrBases.length - 1) {
            nextBase = arrBases[results + 1];
         }

         if(j < arrQualities.length) {
            baseQuality = arrQualities[j] - 33;
         }

         if(j < mapQualities.length) {
            mapQuality = mapQualities[j] - 33;
         }

         String strands1;
         if((readBase.equals(".") || readBase.equals(",")) && !nextBase.equals("-") && !nextBase.equals("+")) {
            strand = "+";
            if(readBase.equals(",")) {
               strand = "-";
            }

            if(baseQuality >= minAvgQual) {
               ++reads1;
               if(strandsSeen.containsKey("ref")) {
                  strands1 = (String)strandsSeen.get("ref");
                  if(strands1.length() < 2 && !strands1.equals(strand)) {
                     strandsSeen.put("ref", (String)strandsSeen.get("ref") + strand);
                  }
               } else {
                  strandsSeen.put("ref", strand);
               }

               if(strand.equals("+")) {
                  if(readCountsPlus.containsKey("ref")) {
                     readCountsPlus.put("ref", Integer.valueOf(((Integer)readCountsPlus.get("ref")).intValue() + 1));
                  } else {
                     readCountsPlus.put("ref", Integer.valueOf(1));
                  }
               } else if(readCountsMinus.containsKey("ref")) {
                  readCountsMinus.put("ref", Integer.valueOf(((Integer)readCountsMinus.get("ref")).intValue() + 1));
               } else {
                  readCountsMinus.put("ref", Integer.valueOf(1));
               }

               if(qualitySum.containsKey("ref")) {
                  qualitySum.put("ref", Integer.valueOf(((Integer)qualitySum.get("ref")).intValue() + baseQuality));
                  mapQualitySum.put("ref", Integer.valueOf(((Integer)mapQualitySum.get("ref")).intValue() + mapQuality));
               } else {
                  qualitySum.put("ref", Integer.valueOf(baseQuality));
                  mapQualitySum.put("ref", Integer.valueOf(mapQuality));
               }
            }

            ++j;
            readStart = false;
         } else if(!readBase.toUpperCase().equals("A") && !readBase.toUpperCase().equals("C") && !readBase.toUpperCase().equals("G") && !readBase.toUpperCase().equals("T")) {
            if(!readBase.equals("+") && !readBase.equals("-")) {
               if(readBase.toUpperCase().equals("N")) {
                  ++j;
               } else if(readBase.equals("^")) {
                  ++results;
                  readStart = true;
               } else if(readBase.equals("$")) {
                  readStart = false;
               } else if(!readBase.equals(".") && !readBase.equals(",")) {
                  ++j;
               }
            } else {
               strands1 = "";
               if(readBase.equals("+")) {
                  strands1 = "INS";
               } else {
                  strands1 = "DEL";
               }

               if((prevBase.equals(".") || prevBase.equals(",")) && prevBaseQuality >= minAvgQual) {
                  ++reads1indel;
               }

               boolean avgQual1 = false;
               boolean avgMapQual1 = true;
               String reads1plus = "";

               String reads1minus;
               int variantKeys;
               try {
                  reads1minus = arrBases[results + 1] + arrBases[results + 2] + arrBases[results + 3];
                  reads1minus = reads1minus.replaceAll("[^0-9]", "");
                  var44 = Integer.parseInt(reads1minus);
                  var47 = var44 + Integer.toString(var44).length();

                  for(variantKeys = 0; variantKeys < var47; ++variantKeys) {
                     key = arrBases[results + 1 + variantKeys];

                     try {
                        Integer.parseInt(key);
                     } catch (Exception var42) {
                        if(!key.equals(".") && !key.equals(",")) {
                           if(key.toUpperCase().equals("A") || key.toUpperCase().equals("C") || key.toUpperCase().equals("G") || key.toUpperCase().equals("T") || key.toUpperCase().equals("N")) {
                              reads1plus = reads1plus + key;
                           }
                        } else {
                           variantKeys = var47;
                        }
                     }
                  }

                  results += var47;
               } catch (Exception var43) {
                  var44 = Integer.parseInt(arrBases[results + 1]);

                  for(variantKeys = 0; variantKeys < var44; ++variantKeys) {
                     reads1plus = reads1plus + arrBases[results + 2 + variantKeys];
                  }

                  results = results + 1 + var44;
               }

               if(reads1plus.equals(reads1plus.toUpperCase())) {
                  strand = "+";
               } else {
                  strand = "-";
               }

               reads1plus = reads1plus.toUpperCase();
               reads1minus = strands1 + "-" + var44 + "-" + reads1plus;
               if(readCounts.containsKey(reads1minus)) {
                  readCounts.put(reads1minus, Integer.valueOf(((Integer)readCounts.get(reads1minus)).intValue() + 1));
               } else {
                  readCounts.put(reads1minus, Integer.valueOf(1));
               }

               if(strand.equals("+")) {
                  if(readCountsPlus.containsKey(reads1minus)) {
                     readCountsPlus.put(reads1minus, Integer.valueOf(((Integer)readCountsPlus.get(reads1minus)).intValue() + 1));
                  } else {
                     readCountsPlus.put(reads1minus, Integer.valueOf(1));
                  }
               } else if(readCountsMinus.containsKey(reads1minus)) {
                  readCountsMinus.put(reads1minus, Integer.valueOf(((Integer)readCountsMinus.get(reads1minus)).intValue() + 1));
               } else {
                  readCountsMinus.put(reads1minus, Integer.valueOf(1));
               }

               if(strandsSeen.containsKey(reads1minus)) {
                  String var50 = (String)strandsSeen.get(reads1minus);
                  if(var50.length() < 2 && !var50.equals(strand)) {
                     strandsSeen.put(reads1minus, (String)strandsSeen.get(reads1minus) + strand);
                  }
               } else {
                  strandsSeen.put(reads1minus, strand);
               }

               if(j < arrQualities.length) {
                  baseQuality = arrQualities[j] - 33;
                  ++j;
               }

               if(j < mapQualities.length) {
                  mapQuality = mapQualities[j] - 33;
               }

               if(qualitySum.containsKey(reads1minus)) {
                  qualitySum.put(reads1minus, Integer.valueOf(((Integer)qualitySum.get(reads1minus)).intValue() + baseQuality));
                  mapQualitySum.put(reads1minus, Integer.valueOf(((Integer)mapQualitySum.get(reads1minus)).intValue() + mapQuality));
               } else {
                  qualitySum.put(reads1minus, Integer.valueOf(baseQuality));
                  mapQualitySum.put(reads1minus, Integer.valueOf(mapQuality));
               }

               readStart = false;
            }
         } else {
            strand = "+";
            if(readBase.equals("a") || readBase.equals("c") || readBase.equals("g") || readBase.equals("t")) {
               strand = "-";
            }

            readBase = readBase.toUpperCase();
            if(baseQuality >= minAvgQual) {
               if(readCounts.containsKey(readBase)) {
                  readCounts.put(readBase, Integer.valueOf(((Integer)readCounts.get(readBase)).intValue() + 1));
               } else {
                  readCounts.put(readBase, Integer.valueOf(1));
               }

               if(strand.equals("+")) {
                  if(readCountsPlus.containsKey(readBase)) {
                     readCountsPlus.put(readBase, Integer.valueOf(((Integer)readCountsPlus.get(readBase)).intValue() + 1));
                  } else {
                     readCountsPlus.put(readBase, Integer.valueOf(1));
                  }
               } else if(readCountsMinus.containsKey(readBase)) {
                  readCountsMinus.put(readBase, Integer.valueOf(((Integer)readCountsMinus.get(readBase)).intValue() + 1));
               } else {
                  readCountsMinus.put(readBase, Integer.valueOf(1));
               }

               if(strandsSeen.containsKey(readBase)) {
                  strands1 = (String)strandsSeen.get(readBase);
                  if(strands1.length() < 2 && !strands1.equals(strand)) {
                     strandsSeen.put(readBase, (String)strandsSeen.get(readBase) + strand);
                  }
               } else {
                  strandsSeen.put(readBase, strand);
               }

               if(qualitySum.containsKey(readBase)) {
                  qualitySum.put(readBase, Integer.valueOf(((Integer)qualitySum.get(readBase)).intValue() + baseQuality));
                  mapQualitySum.put(readBase, Integer.valueOf(((Integer)mapQualitySum.get(readBase)).intValue() + mapQuality));
               } else {
                  qualitySum.put(readBase, Integer.valueOf(baseQuality));
                  mapQualitySum.put(readBase, Integer.valueOf(mapQuality));
               }
            }

            ++j;
            readStart = false;
         }
      }

      HashMap var45 = new HashMap();
      int var46 = 0;
      if(strandsSeen.containsKey("ref")) {
         var46 = ((String)strandsSeen.get("ref")).length();
      }

      var44 = 0;
      if(reads1 > 0) {
         var44 = ((Integer)qualitySum.get("ref")).intValue() / reads1;
      }

      var47 = 0;
      if(reads1 > 0) {
         var47 = ((Integer)mapQualitySum.get("ref")).intValue() / reads1;
      }

      int var48 = 0;
      int var49 = 0;
      if(readCountsPlus.containsKey("ref")) {
         var48 = ((Integer)readCountsPlus.get("ref")).intValue();
      }

      if(readCountsMinus.containsKey("ref")) {
         var49 = ((Integer)readCountsMinus.get("ref")).intValue();
      }

      if(reads1 < 0) {
         reads1 = 0;
      }

      var45.put(refBase, reads1 + "\t" + var46 + "\t" + var44 + "\t" + var47 + "\t" + var48 + "\t" + var49 + "\t" + reads1indel);
      String[] var51 = (String[])readCounts.keySet().toArray(new String[0]);
      Arrays.sort(var51);
      String[] var35 = var51;
      int var34 = var51.length;

      for(int e = 0; e < var34; ++e) {
         key = var35[e];
         int reads2 = ((Integer)readCounts.get(key)).intValue();
         int reads2plus = 0;
         int reads2minus = 0;
         if(readCountsPlus.containsKey(key)) {
            reads2plus = ((Integer)readCountsPlus.get(key)).intValue();
         }

         if(readCountsMinus.containsKey(key)) {
            reads2minus = ((Integer)readCountsMinus.get(key)).intValue();
         }

         int strands2 = 0;
         if(strandsSeen.containsKey(key)) {
            strands2 = ((String)strandsSeen.get(key)).length();
         }

         int avg_qual2 = ((Integer)qualitySum.get(key)).intValue() / reads2;
         int avg_map_qual2 = ((Integer)mapQualitySum.get(key)).intValue() / reads2;
         if(reads2 > 0) {
            var45.put(key, reads2 + "\t" + strands2 + "\t" + avg_qual2 + "\t" + avg_map_qual2 + "\t" + reads2plus + "\t" + reads2minus);
         }
      }

      return var45;
   }

   static int qualityDepth(String readQuals, int minAvgQual) {
      boolean baseQuality = false;
      int qualityDepth = 0;
      char[] arrQualities = readQuals.toCharArray();
      boolean j = false;

      for(int var7 = 0; var7 < arrQualities.length; ++var7) {
         int var6 = arrQualities[var7] - 33;
         if(var6 >= minAvgQual) {
            ++qualityDepth;
         }
      }

      return qualityDepth;
   }

   static String callPosition(String refBase, HashMap readCounts, String callType, int minReads2, double minVarFreq, int minAvgQual, double pValueThreshold, double minFreqForHom) {
      String callResult = "";
      DecimalFormat df = new DecimalFormat("###.##");
      int reads1 = 0;
      int reads2 = 0;
      int readsWithIndels = 0;
      int strands1 = 0;
      int strands2 = 0;
      int avgQual1 = 0;
      int avgQual2 = 0;
      int avgMap1 = 0;
      int avgMap2 = 0;
      int reads1indel = 0;
      int reads1plus = 0;
      int reads1minus = 0;
      int reads2plus = 0;
      int reads2minus = 0;
      double pValue = 1.0D;
      double varFreq = 0.0D;
      String varAllele = "";

      try {
         String[] e;
         if(readCounts.containsKey(refBase)) {
            try {
               e = ((String)readCounts.get(refBase)).split("\t");
               reads1 = Integer.parseInt(e[0]);
               strands1 = Integer.parseInt(e[1]);
               avgQual1 = Integer.parseInt(e[2]);
               avgMap1 = Integer.parseInt(e[3]);
               reads1plus = Integer.parseInt(e[4]);
               reads1minus = Integer.parseInt(e[5]);
               if(e.length > 6) {
                  reads1indel = Integer.parseInt(e[6]);
               }
            } catch (Exception var53) {
               System.err.println("Error parsing refBase readcounts from " + (String)readCounts.get(refBase));
            }
         }

         e = (String[])readCounts.keySet().toArray(new String[0]);
         Arrays.sort(e);
         int totalReadCounts = 0;
         String[] var37 = e;
         int var36 = e.length;

         String allele;
         int var35;
         String[] alleleContents;
         int thisReads1;
         for(var35 = 0; var35 < var36; ++var35) {
            allele = var37[var35];
            alleleContents = ((String)readCounts.get(allele)).split("\t");

            try {
               thisReads1 = Integer.parseInt(alleleContents[0]);
               totalReadCounts += thisReads1;
            } catch (Exception var52) {
               ;
            }
         }

         var37 = e;
         var36 = e.length;

         for(var35 = 0; var35 < var36; ++var35) {
            allele = var37[var35];
            alleleContents = ((String)readCounts.get(allele)).split("\t");
            if(!allele.equals(refBase)) {
               int thisReads2 = 0;
               int thisStrands2 = 0;
               int thisAvgQual2 = 0;
               int thisAvgMap2 = 0;
               int thisReads2plus = 0;
               int thisReads2minus = 0;

               try {
                  thisReads2 = Integer.parseInt(alleleContents[0]);
                  thisStrands2 = Integer.parseInt(alleleContents[1]);
                  thisAvgQual2 = Integer.parseInt(alleleContents[2]);
                  thisAvgMap2 = Integer.parseInt(alleleContents[3]);
                  thisReads2plus = Integer.parseInt(alleleContents[4]);
                  thisReads2minus = Integer.parseInt(alleleContents[5]);
                  if(allele.contains("INS") || allele.contains("DEL")) {
                     readsWithIndels += thisReads2;
                  }
               } catch (Exception var54) {
                  ;
               }

               if(!callType.equals("CNS") || thisReads2 > reads2) {
                  double thisVarFreq = (double)thisReads2 / (double)totalReadCounts;
                  double thisPvalue = 1.0D;
                  if(allele.contains("INS") || allele.contains("DEL")) {
                     int thisVarType = totalReadCounts - reads1indel;
                     if(thisVarType < thisReads2) {
                        ;
                     }

                     thisVarFreq = (double)thisReads2 / (double)totalReadCounts;
                  }

                  if(pValueThreshold == 0.99D) {
                     thisPvalue = 0.98D;
                  } else {
                     thisPvalue = getSignificance(reads1, thisReads2);
                  }

                  if(thisReads2 > reads2 && thisAvgQual2 >= minAvgQual) {
                     if(!allele.contains("INS") && !allele.contains("DEL")) {
                        varAllele = allele;
                     } else {
                        varAllele = getShortIndel(allele);
                     }

                     reads2 = thisReads2;
                     strands2 = thisStrands2;
                     avgQual2 = thisAvgQual2;
                     avgMap2 = thisAvgMap2;
                     reads2plus = thisReads2plus;
                     reads2minus = thisReads2minus;
                     varFreq = thisVarFreq * 100.0D;
                     pValue = thisPvalue;
                  }

                  if(thisReads2 >= minReads2 && thisAvgQual2 >= minAvgQual && thisVarFreq >= minVarFreq) {
                     thisReads1 = reads1;
                     thisVarFreq *= 100.0D;
                     String var56 = "SNP";
                     if(allele.contains("INS") || allele.contains("DEL")) {
                        var56 = "INDEL";
                        thisReads1 = reads1;
                        if(reads1 < 0) {
                           thisReads1 = 0;
                        }

                        allele = getShortIndel(allele);
                     }

                     if(thisPvalue <= pValueThreshold) {
                        String genotype;
                        if(!callType.equals("SNP") && !callType.equals("INDEL")) {
                           if(callType.equals("CNS") && thisReads2 >= reads2) {
                              reads2 = thisReads2;
                              strands2 = thisStrands2;
                              avgQual2 = thisAvgQual2;
                              avgMap2 = thisAvgMap2;
                              reads2plus = thisReads2plus;
                              reads2minus = thisReads2minus;
                              pValue = thisPvalue;
                              genotype = "";
                              if(thisVarFreq >= minFreqForHom * 100.0D) {
                                 genotype = allele + allele;
                                 if(var56.equals("INDEL")) {
                                    genotype = allele + "/" + allele;
                                 }
                              } else {
                                 genotype = refBase + allele;
                                 if(var56.equals("INDEL")) {
                                    genotype = "*/" + allele;
                                 }
                              }

                              callResult = genotypeToCode(genotype) + "\t" + thisReads1 + "\t" + thisReads2 + "\t" + df.format(thisVarFreq) + "%\t" + strands1 + "\t" + thisStrands2 + "\t" + avgQual1 + "\t" + thisAvgQual2 + "\t" + thisPvalue;
                              callResult = callResult + "\t" + avgMap1 + "\t" + thisAvgMap2;
                              callResult = callResult + "\t" + reads1plus + "\t" + reads1minus + "\t" + thisReads2plus + "\t" + thisReads2minus + "\t" + varAllele;
                           }
                        } else {
                           reads2 = thisReads2;
                           strands2 = thisStrands2;
                           avgQual2 = thisAvgQual2;
                           avgMap2 = thisAvgMap2;
                           reads2plus = thisReads2plus;
                           reads2minus = thisReads2minus;
                           pValue = thisPvalue;
                           genotype = "";
                           if(thisVarFreq >= minFreqForHom * 100.0D) {
                              genotype = allele + allele;
                              if(var56.equals("INDEL")) {
                                 genotype = allele + "/" + allele;
                              }
                           } else {
                              genotype = refBase + allele;
                              if(var56.equals("INDEL")) {
                                 genotype = "*/" + allele;
                              }
                           }

                           if(var56.equals(callType)) {
                              if(callResult.length() > 0) {
                                 callResult = callResult + "\n";
                              }

                              if(thisReads1 < 0) {
                                 thisReads1 = 0;
                              }

                              if(thisReads2 < 0) {
                                 reads2 = 0;
                              }

                              callResult = callResult + genotypeToCode(genotype) + "\t" + thisReads1 + "\t" + reads2 + "\t" + df.format(thisVarFreq) + "%\t" + strands1 + "\t" + thisStrands2 + "\t" + avgQual1 + "\t" + thisAvgQual2 + "\t" + thisPvalue;
                              callResult = callResult + "\t" + avgMap1 + "\t" + thisAvgMap2;
                              callResult = callResult + "\t" + reads1plus + "\t" + reads1minus + "\t" + thisReads2plus + "\t" + thisReads2minus + "\t" + varAllele;
                           }
                        }
                     }
                  }
               }
            }
         }
      } catch (Exception var55) {
         System.err.println("Read Counts Exception: " + var55.getLocalizedMessage());
         var55.printStackTrace(System.err);
      }

      if(callResult.length() == 0 && callType.equals("CNS")) {
         if(reads1 > 0 && reads1 > minReads2) {
            callResult = refBase + "\t" + reads1 + "\t" + reads2 + "\t" + df.format(varFreq) + "%\t" + strands1 + "\t" + strands2 + "\t" + avgQual1 + "\t" + avgQual2 + "\t" + pValue;
            callResult = callResult + "\t" + avgMap1 + "\t" + avgMap2;
            callResult = callResult + "\t" + reads1plus + "\t" + reads1minus + "\t" + reads2plus + "\t" + reads2minus + "\t" + varAllele;
         } else {
            callResult = "N\t" + reads1 + "\t" + reads2 + "\t" + df.format(varFreq) + "%\t" + strands1 + "\t" + strands2 + "\t" + avgQual1 + "\t" + avgQual2 + "\t" + pValue;
            callResult = callResult + "\t" + avgMap1 + "\t" + avgMap2;
            callResult = callResult + "\t" + reads1plus + "\t" + reads1minus + "\t" + reads2plus + "\t" + reads2minus + "\t" + varAllele;
         }
      }

      return callResult;
   }

   static String strandFilter(int reads1plus, int reads1minus, int reads2plus, int reads2minus, double strandPvalueThreshold) {
      DecimalFormat pvalueFormat = new DecimalFormat("0.####E0");
      String strandFilterStatus = "Pass:" + reads1plus + ":" + reads1minus + ":" + reads2plus + ":" + reads2minus + ":" + 1;
      double refStrandPlus = 0.5D;
      double varStrandPlus = 0.5D;
      double strandPvalue = 1.0D;
      if(reads2plus + reads2minus > 0) {
         varStrandPlus = (double)reads2plus / (double)(reads2plus + reads2minus);
      }

      if(varStrandPlus < 0.1D || varStrandPlus > 0.9D) {
         if(reads1plus + reads1minus > 1) {
            refStrandPlus = (double)reads1plus / (double)(reads1plus + reads1minus);
            strandPvalue = getSignificance(reads1plus, reads1minus, reads2plus, reads2minus);
         } else {
            int testReads1plus = (reads2plus + reads2minus) / 2;
            int testReads1minus = reads2plus + reads2minus - testReads1plus;
            strandPvalue = getSignificance(testReads1plus, testReads1minus, reads2plus, reads2minus);
         }

         strandFilterStatus = "Pass:" + varStrandPlus + ":" + reads1plus + ":" + reads1minus + ":" + reads2plus + ":" + reads2minus + ":" + pvalueFormat.format(strandPvalue);
         if(refStrandPlus >= 0.1D && refStrandPlus <= 0.9D && (varStrandPlus < 0.1D || varStrandPlus > 0.9D) && strandPvalue < strandPvalueThreshold) {
            strandFilterStatus = "Fail:" + reads1plus + ":" + reads1minus + ":" + reads2plus + ":" + reads2minus + ":" + pvalueFormat.format(strandPvalue);
         }
      }

      return strandFilterStatus;
   }

   public static double getSignificance(int obsReads1, int obsReads2) {
      double pValue = 1.0D;
      double baseline_error = 0.001D;
      int coverage = obsReads1 + obsReads2;
      int expReads2 = (int)((double)coverage * baseline_error);
      int expReads1 = coverage - expReads2;
      pValue = getSignificance(expReads1, expReads2, obsReads1, obsReads2);
      return pValue;
   }

   public static double getSignificance(int expReads1, int expReads2, int obsReads1, int obsReads2) {
      double pValue = 1.0D;
      if(expReads1 < 0) {
         expReads1 = 0;
      }

      if(expReads2 < 0) {
         expReads2 = 0;
      }

      if(obsReads1 < 0) {
         obsReads1 = 0;
      }

      if(obsReads2 < 0) {
         obsReads2 = 0;
      }

      FishersExact fisher = new FishersExact(expReads1 + expReads2 + obsReads1 + obsReads2 + 100);
      pValue = fisher.getRightTailedP(expReads1, expReads2, obsReads1, obsReads2);
      int fisher_max = 1000;

      int num_tries;
      for(num_tries = 0; Double.isNaN(pValue) && num_tries < 10; ++num_tries) {
         fisher = new FishersExact(expReads1 + expReads2 + obsReads1 + obsReads2 + fisher_max);
         pValue = fisher.getRightTailedP(expReads1, expReads2, obsReads1, obsReads2);
         fisher_max += 1000;
      }

      if(num_tries >= 10) {
         System.err.println("Warning: unable to calculate p-value failure: " + expReads1 + "," + expReads2 + "," + obsReads1 + "," + obsReads2);
      }

      if(pValue >= 0.999D) {
         for(pValue = fisher.getLeftTailedP(expReads1, expReads2, obsReads1, obsReads2); Double.isNaN(pValue); fisher_max += 1000) {
            fisher = new FishersExact(expReads1 + expReads2 + obsReads1 + obsReads2 + fisher_max);
            pValue = fisher.getLeftTailedP(expReads1, expReads2, obsReads1, obsReads2);
         }
      }

      return pValue;
   }

   static String genotypeToCode(String gt) {
      return gt.equals("AA")?"A":(gt.equals("CC")?"C":(gt.equals("GG")?"G":(gt.equals("TT")?"T":(!gt.equals("AC") && !gt.equals("CA")?(!gt.equals("AG") && !gt.equals("GA")?(!gt.equals("AT") && !gt.equals("TA")?(!gt.equals("CG") && !gt.equals("GC")?(!gt.equals("CT") && !gt.equals("TC")?(!gt.equals("GT") && !gt.equals("TG")?(gt.substring(0, 1).equals("N")?"N":gt):"K"):"Y"):"S"):"W"):"R"):"M"))));
   }

   static String getVarAllele(String refBase, String consCode) {
      String varAllele = consCode;
      if(consCode.contains("/")) {
         String[] tempArray = consCode.split("/");
         if(tempArray.length > 1) {
            varAllele = tempArray[1];
         }
      } else if(consCode.equals("M") || consCode.equals("R") || consCode.equals("W") || consCode.equals("S") || consCode.equals("Y") || consCode.equals("K")) {
         if(consCode.equals("M")) {
            if(refBase.equals("A")) {
               varAllele = "C";
            } else {
               varAllele = "A";
            }
         } else if(consCode.equals("R")) {
            if(refBase.equals("A")) {
               varAllele = "G";
            } else {
               varAllele = "A";
            }
         } else if(consCode.equals("W")) {
            if(refBase.equals("A")) {
               varAllele = "T";
            } else {
               varAllele = "A";
            }
         } else if(consCode.equals("S")) {
            if(refBase.equals("G")) {
               varAllele = "C";
            } else {
               varAllele = "G";
            }
         } else if(consCode.equals("Y")) {
            if(refBase.equals("C")) {
               varAllele = "T";
            } else {
               varAllele = "C";
            }
         } else if(consCode.equals("K")) {
            if(refBase.equals("G")) {
               varAllele = "T";
            } else {
               varAllele = "G";
            }
         }
      }

      return varAllele;
   }

   static String getShortIndel(String gt) {
      if(gt.contains("INS") || gt.contains("DEL")) {
         try {
            String[] e = gt.split("-");
            String indel_type = e[0];
            String indel_size = e[1];
            String indel_bases = e[2];
            if(indel_type.contains("INS")) {
               return "+" + indel_bases;
            }

            return "-" + indel_bases;
         } catch (Exception var5) {
            System.err.println("Warning: error generating consensus from " + gt);
         }
      }

      return "N";
   }

   static boolean isHeterozygous(String genotype) {
      if(genotype.contains("/")) {
         String[] alleles = genotype.split("/");
         if(!alleles[0].equals(alleles[1])) {
            return true;
         }
      } else if(genotype.equals("M") || genotype.equals("R") || genotype.equals("W") || genotype.equals("S") || genotype.equals("Y") || genotype.equals("K")) {
         return true;
      }

      return false;
   }

   static boolean isHomozygous(String genotype) {
      if(genotype.contains("/")) {
         String[] alleles = genotype.split("/");
         if(alleles[0].equals(alleles[1])) {
            return true;
         }
      } else if(genotype.equals("A") || genotype.equals("C") || genotype.equals("G") || genotype.equals("T")) {
         return true;
      }

      return false;
   }

   static String codeToGenotype(String code) {
      return !code.equals("A") && !code.equals("C") && !code.equals("G") && !code.equals("T")?(code.contains("/")?code:(code.equals("M")?"A/C":(code.equals("R")?"A/G":(code.equals("W")?"A/T":(code.equals("S")?"C/G":(code.equals("Y")?"C/T":(code.equals("K")?"G/T":"N/N"))))))):code + "/" + code;
   }
}
